/*----------------------------------------------------------------------------*
 | Sourcefile Test.h                                                          |
 |                                                                            |
 | Version 2.00                                                               |
 |                                                                            |
 | Description:                                                               |
 |      Just a crappy test program to test the callbacks.                     |
 |                                                                            |
 |                                                                            |
 | This file is part of the the Bramz::BriDGe library.  The Bramz::BriDGe     |
 | library is about stuff that is developed for my own purpose, for having a  |
 | solid code base for my software.  However this stuff might get distributed |
 | somehow and used in other places than I originally intented.               |
 | Which is nice ...                                                          |
 |                                                                            |
 |                                                                            |
 | Copyright (C) 2002 Bram de Greve                                           |
 |                                                                            |
 |                                                                            |
 | This file is free software; you can redistribute it and/or modify          |
 | it under the terms of the GNU General Public License as published by       |
 | the Free Software Foundation; either version 2 of the License, or          |
 | (at your option) any later version.                                        |
 |                                                                            |
 | This file is distributed in the hope that it will be useful,               |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of             |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              |
 | GNU General Public License for more details.                               |
 |                                                                            |
 | You should have received a copy of the GNU General Public License          |
 | along with this file; if not, write to the Free Software                   |
 | Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  |
 |                                                                            |
 |                                                                            |
 | Contact by e-mail: Bram de Greve <bdegreve@mail.be>                        |
 |                                                                            |
 |                                                                            |
 | History:                                                                   |
 | June 7th, 2002: creation of Test.cpp                                       |
 | June 9th, 2002: Updating to test the BriDGe::Callback library 2.00         |
 |                                                                            |
 | I want to thank Altair, Jaap Suter & especially Dirk Gerrits (Who has      |
 | spent an entire afternoon on this to help me.  It was a waste of time for  |
 | both of us, but we had a lot of fun, didn't we? :)                         |
 *----------------------------------------------------------------------------*/

#include "Callback0.h"
#include "Callback1.h"
#include "CallbackR0.h"
#include "CallbackR1.h"

#include <string>
#include <iostream>

// for once, we'll take shortcuts
using namespace BDG;
using namespace std;



// --- TEST MATERIAL -----------------------------------------------------------

class Dog
{
public:
    Dog(const string& a_name, int a_age, int a_pos): m_name(a_name), m_age(a_age), m_pos(a_pos) {}

    // 0
    void birthday() { ++m_age; cout << m_name << " is " << m_age << " now." << endl; }
    void bark() const { cout << m_name << ": Whoof!\n"; }
    
    // R0
    int goCrazy() { m_pos = rand(); return m_pos; }
    int getAge() const { return m_age; }

    // 1
    void setAge(int a_age) { m_age = a_age; cout << "m_age: " << m_age << endl; }
    void jump(int a_height) const { cout << "Jump " << a_height << " feet high.\n"; }

    // R1
    int run(int a_distance) { m_pos += a_distance; return m_pos; }
    bool call(const string& a_name) const { return m_name == a_name; }

private:
    string m_name;
    int m_pos;
    int m_age;
};

void foo0() { cout << "foo0.\n"; }
void foo1(int a_i) { cout << "foo1: " << a_i << endl; }
int fooR0() { return 999; }
bool fooR1(const string& a_s) { return a_s.empty(); }



// --- THE TEST CHAMBER --------------------------------------------------------

int main()
{
    Dog eddy("Eddy", 2, 100);
    Dog freddy("Freddy", 3, 1000);


    // CALLBACK0

    Callback0 cb1;
    Callback0 cb2(&eddy, Dog::birthday);
    Callback0 cb3(&freddy, Dog::bark);

    if (!cb1.empty())
    {
        cb1();
    }
    else
    {
        cout << "Saved by the bell!\n";
    }

    try
    {
        cb1();
    }
    catch (CallbackEmptyCall emptyCall)
    {
        cout << emptyCall.what() << endl;
    }

    cb2();
    cb3();
    Callback0 cb4(cb2);
    cb4();
    cb1.swap(cb3);
    cb1();
    cb1 = Callback0(foo0);
    cb1();
    
    cout << endl;
    

    // CALLBACKR0

    CallbackR0<int> cb5;
    CallbackR0<int> cb6(&eddy, Dog::goCrazy);
    CallbackR0<int> cb7(&freddy, Dog::getAge);

    if (!cb5.empty())
    {
        cb5();
    }
    else
    {
        cout << "Saved by the bell!\n";
    }

    try
    {
        cb5();
    }
    catch (CallbackEmptyCall emptyCall)
    {
        cout << emptyCall.what() << endl;
    }

    cout << "Eddy is now at: " << cb6() << endl;
    cout << "Freddy is now " << cb7() << " years old.\n";
    CallbackR0<int> cb8(cb6);
    cout << "Eddy goes crazy again: " << cb8() << endl;
    cb5.swap(cb7);
    cout << "Age of Freddy again: " << cb5() << endl;
    cb5 = CallbackR0<int>(fooR0);
    cout << "Number: " << cb5() << endl;

    cout << endl;
    char cont;
    cout << "Press enter to test the rest ...";
    cin.get(cont);
    cout << endl;


    // CALLBACK1

    Callback1<int> cb9;
    Callback1<int> cb10(&eddy, Dog::setAge);
    Callback1<int> cb11(&freddy, Dog::jump);

    if (!cb9.empty())
    {
        cb9(666);
    }
    else
    {
        cout << "Saved by the bell!\n";
    }

    try
    {
        cb9(666);
    }
    catch (CallbackEmptyCall emptyCall)
    {
        cout << emptyCall.what() << endl;
    }

    cb10(10);
    cb11(100);
    Callback1<int> cb12(cb10);
    cb12(20);
    cb9.swap(cb11);
    cb9(200);
    cb9 = Callback1<int>(foo1);
    cb9(9);

    cout << endl;
        

    // CALLBACKR1

    CallbackR1<bool, const string&> cb13;
    CallbackR1<int, int> cb14(&eddy, Dog::run);
    CallbackR1<bool, const string&> cb15(&freddy, Dog::call);

    if (!cb13.empty())
    {
        cb13("666");
    }
    else
    {
        cout << "Saved by the bell!\n";
    }

    try
    {
        cb13("666");
    }
    catch (CallbackEmptyCall emptyCall)
    {
        cout << emptyCall.what() << endl;
    }

    cout << "Eddy runs and ends at " << cb14(10) << endl;
    cout << "Freddy! " << (cb15("Freddy") ? "Whoof!" : "tumbleweed") << endl;
    CallbackR1<int, int> cb16(cb14);
    cout << "Eddy runs again: " << cb16(20) << endl;
    cb13.swap(cb15);
    cout << "Josper! " << (cb13("Josper") ? "Whoof!" : "tumbleweed") << endl;
    cb13 = CallbackR1<bool, const std::string&>(fooR1);
    cout << "Is Franske empty? " << cb13("Franske") << endl;

    return 0;
}



// --- END OF FILE Test.cpp ----------------------------------------------------
